@@ -30,6 +30,7 @@
 #include <errno.h>
 #include <arpa/inet.h>
 #include <sys/types.h>
+#include <sys/wait.h> 
 #include <unistd.h>
 #include <glib.h>
 
@@ -107,16 +108,15 @@ merge_one_ip4_config (NMIP4Config *dst, NMIP4Config *src)
 		nm_ip4_config_add_domain (dst, nm_ip4_config_get_domain (src, i));
 
 	num = nm_ip4_config_get_num_searches (src);
-	if (num > 0) {
-		for (i = 0; i < num; i++)
-			nm_ip4_config_add_search (dst, nm_ip4_config_get_search (src, i));
-	} else {
-		/* If no search domains were specified, add the 'domain' list to
-		 * search domains.
-		 */
-		for (i = 0; i < num_domains; i++)
-			nm_ip4_config_add_search (dst, nm_ip4_config_get_domain (src, i));
-	}
+	for (i = 0; i < num; i++)
+		nm_ip4_config_add_search (dst, nm_ip4_config_get_search (src, i));
+
+	/* Add the 'domain' list to searches as well since overloading the
+	 * 'domain_name' DHCP field used to be the way you got searches
+	 * into resolv.conf.
+	 */
+	for (i = 0; i < num_domains; i++)
+		nm_ip4_config_add_search (dst, nm_ip4_config_get_domain (src, i));
 }
 
 
@@ -131,12 +131,12 @@ netconfig_child_setup (gpointer user_data G_GNUC_UNUSED)
 	setpgid (pid, pid);
 }
 
-static gint
-run_netconfig (GError **error)
+static GPid
+run_netconfig (GError **error, gint *stdin_fd)
 {
 	char *argv[5];
 	char *tmp;
-	gint stdin_fd;
+	GPid pid = -1;
 
 	argv[0] = "/sbin/netconfig";
 	argv[1] = "modify";
@@ -149,10 +149,10 @@ run_netconfig (GError **error)
 	g_free (tmp);
 
 	if (!g_spawn_async_with_pipes (NULL, argv, NULL, 0, netconfig_child_setup,
-	                               NULL, NULL, &stdin_fd, NULL, NULL, error))
+	                               NULL, &pid, stdin_fd, NULL, NULL, error))
 		return -1;
 
-	return stdin_fd;
+	return pid;
 }
 
 static void
@@ -169,16 +169,18 @@ write_to_netconfig (gint fd, const char *key, const char *value)
 
 static gboolean
 dispatch_netconfig (const char *domain,
-				char **searches,
-				char **nameservers,
-				const char *iface,
-				GError **error)
+                    char **searches,
+                    char **nameservers,
+                    const char *iface,
+                    GError **error)
 {
-	gint fd;
 	char *str;
+	GPid pid;
+	gint fd;
+	int ret;
 
-	fd = run_netconfig (error);
-	if (fd < 0)
+	pid = run_netconfig (error, &fd);
+	if (pid < 0)
 		return FALSE;
 
 	write_to_netconfig (fd, "INTERFACE", iface);
@@ -206,16 +208,24 @@ dispatch_netconfig (const char *domain,
 
 	close (fd);
 
-	return TRUE;
+	/* Wait until the process exits */
+
+ again:
+
+	ret = waitpid (pid, NULL, 0);
+	if (ret < 0 && errno == EINTR)
+		goto again;
+
+	return ret > 0;
 }
 #endif
 
 
 static gboolean
 write_resolv_conf (FILE *f, const char *domain,
-		   char **searches,
-		   char **nameservers,
-		   GError **error)
+                   char **searches,
+                   char **nameservers,
+                   GError **error)
 {
 	char *domain_str = NULL;
 	char *searches_str = NULL;
@@ -283,10 +293,10 @@ write_resolv_conf (FILE *f, const char *domain,
 #ifdef RESOLVCONF_PATH
 static gboolean
 dispatch_resolvconf (const char *domain,
-		     char **searches,
-		     char **nameservers,
-		     const char *iface,
-		     GError **error)
+                     char **searches,
+                     char **nameservers,
+                     const char *iface,
+                     GError **error)
 {
 	char *cmd;
 	FILE *f;
@@ -324,26 +334,35 @@ dispatch_resolvconf (const char *domain,
 
 static gboolean
 update_resolv_conf (const char *domain,
-		    		char **searches,
-				char **nameservers,
-				const char *iface,
-				GError **error)
+                    char **searches,
+                    char **nameservers,
+                    const char *iface,
+                    GError **error)
 {
 	const char *tmp_resolv_conf = RESOLV_CONF ".tmp";
 	char tmp_resolv_conf_realpath [PATH_MAX];
 	FILE *f;
+	int do_rename = 1;
+	int old_errno = 0;
 
-	if (!realpath(tmp_resolv_conf, tmp_resolv_conf_realpath))
-		strcpy(tmp_resolv_conf_realpath, tmp_resolv_conf);
+	if (!realpath (tmp_resolv_conf, tmp_resolv_conf_realpath))
+		strcpy (tmp_resolv_conf_realpath, tmp_resolv_conf);
 
 	if ((f = fopen (tmp_resolv_conf_realpath, "w")) == NULL) {
-		g_set_error (error,
-				   NM_NAMED_MANAGER_ERROR,
-				   NM_NAMED_MANAGER_ERROR_SYSTEM,
-				   "Could not open %s: %s\n",
-				   tmp_resolv_conf_realpath,
-				   g_strerror (errno));
-		return FALSE;
+		do_rename = 0;
+		old_errno = errno;
+		if ((f = fopen (RESOLV_CONF, "w")) == NULL) {
+			g_set_error (error,
+			             NM_NAMED_MANAGER_ERROR,
+			             NM_NAMED_MANAGER_ERROR_SYSTEM,
+			             "Could not open %s: %s\nCould not open %s: %s\n",
+			             tmp_resolv_conf_realpath,
+			             g_strerror (old_errno),
+			             RESOLV_CONF,
+			             g_strerror (errno));
+			return FALSE;
+		}
+		strcpy (tmp_resolv_conf_realpath, RESOLV_CONF);
 	}
 
 	write_resolv_conf (f, domain, searches, nameservers, error);
@@ -359,7 +378,7 @@ update_resolv_conf (const char *domain,
 		}
 	}
 
-	if (*error == NULL) {
+	if (*error == NULL && do_rename) {
 		if (rename (tmp_resolv_conf, RESOLV_CONF) < 0) {
 			g_set_error (error,
 					   NM_NAMED_MANAGER_ERROR,
@@ -372,6 +391,35 @@ update_resolv_conf (const char *domain,
 	return *error ? FALSE : TRUE;
 }
 
+static char **
+compute_searches (guint32 num, NMIP4Config *config, gboolean searches)
+{
+	GPtrArray *array;
+	size_t len, elem_len;
+	const char *elem;
+	int i;
+
+	/* Search list is limited to 6 domains total per 'man resolv.conf' */
+	if (num > 6)
+		num = 6;
+
+	array = g_ptr_array_sized_new (num + 1);
+	for (i = 0, len = 0; i < num; i++) {
+		elem = searches ? nm_ip4_config_get_search (config, i)
+			        : nm_ip4_config_get_domain (config, i);
+		elem_len = strlen (elem);
+
+		/* The search list is limited to 256 characters per 'man resolv.conf' */
+		if (len + elem_len > 255)
+			break;
+
+		g_ptr_array_add (array, g_strdup (elem));
+		len += elem_len + 1;  /* +1 for spaces */
+	}
+
+	g_ptr_array_add (array, NULL);
+	return (char **) g_ptr_array_free (array, FALSE);
+}
 
 static gboolean
 rewrite_resolv_conf (NMNamedManager *mgr, const char *iface, GError **error)
@@ -424,21 +472,10 @@ rewrite_resolv_conf (NMNamedManager *mgr, const char *iface, GError **error)
 		domain = nm_ip4_config_get_domain (composite, 0);
 
 	/* Searches */
-	if (num_searches > 0) {
-		array = g_ptr_array_sized_new (num_searches + 1);
-		for (i = 0; i < num_searches; i++)
-			g_ptr_array_add (array, g_strdup (nm_ip4_config_get_search (composite, i)));
-
-		g_ptr_array_add (array, NULL);
-		searches = (char **) g_ptr_array_free (array, FALSE);
-	} else if (num_domains > 0) {
-		array = g_ptr_array_sized_new (num_domains + 1);
-		for (i = 0; i < num_domains; i++)
-			g_ptr_array_add (array, g_strdup (nm_ip4_config_get_domain (composite, i)));
-
-		g_ptr_array_add (array, NULL);
-		searches = (char **) g_ptr_array_free (array, FALSE);
-	}
+	if (num_searches > 0)
+		searches = compute_searches (num_searches, composite, TRUE);
+	else if (num_domains > 0)
+		searches = compute_searches (num_searches, composite, FALSE);
 
 	/* Name servers */
 	num_nameservers = nm_ip4_config_get_num_nameservers (composite);
@@ -464,6 +501,8 @@ rewrite_resolv_conf (NMNamedManager *mgr, const char *iface, GError **error)
 		nameservers = (char **) g_ptr_array_free (array, FALSE);
 	}
 
+	g_object_unref (composite);
+
 #ifdef RESOLVCONF_PATH
 	success = dispatch_resolvconf (domain, searches, nameservers, iface, error);
 #endif
