@@ -227,7 +227,8 @@ should_ignore_file (const char *filename)
 		&& !check_suffix (base, BAK_TAG)
 		&& !check_suffix (base, TILDE_TAG)
 		&& !check_suffix (base, ORIG_TAG)
-		&& !check_suffix (base, REJ_TAG))
+		&& !check_suffix (base, REJ_TAG)
+		&& !check_suffix (base, RPMNEW_TAG))
 		ignore = FALSE;
 
 	g_free (base);
